/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */

 




/* GPL_HEADER */
package org.greenstone.gatherer.cdm;
/**************************************************************************************
 * Title:        Gatherer
 * Description:  The Gatherer: a tool for gathering and enriching a digital collection.
 * Company:      The University of Waikato
 * Written:      02/05/02
 * Revised:      16/08/02 Optimized and added Destructor.
 **************************************************************************************/
import java.awt.BorderLayout;
import java.awt.Rectangle;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileOutputStream;
import java.lang.Exception;
import java.lang.String;
import java.lang.StringBuffer;
import java.util.ArrayList;
import java.util.StringTokenizer;
import javax.swing.JPanel;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.cdm.ClassifierManager;
import org.greenstone.gatherer.cdm.CollectionMetaManager;
import org.greenstone.gatherer.cdm.FormatManager;
import org.greenstone.gatherer.cdm.GUI;
import org.greenstone.gatherer.cdm.Index;
import org.greenstone.gatherer.cdm.IndexManager;
import org.greenstone.gatherer.cdm.MetadataSetManager;
import org.greenstone.gatherer.cdm.PlugInManager;
import org.greenstone.gatherer.cdm.SubcollectionManager;
import org.greenstone.gatherer.util.EmailAddress;
/** This manager provides access to submanagers, which in turn provide tools for the designing of Greenstone collections via the information stored in etc/collect.cfg. This class acts as a hub for the managers that handle specific parts of the configuration such as classifiers, format strings and language settings.
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3
 */
// ####################################################################################
// Optimization                          Saving
// ####################################################################################
// Vector -> ArrayList                   + Processor
// String -> StringBuffer                + Memory, + Processor
// ####################################################################################
public class CollectionDesignManager {
    /** Whether this collection is to be made public or not. */
    public boolean public_col = false;
    /** Whether this collection is a beta version or not. */
    public boolean beta = true;
    /** A list of classifiers to use at build time. */
    public ClassifierManager classifiers = null;
    /** A manager of collection level metadata. */
    public CollectionMetaManager collectionmetadatum = null;
    /** E-mail address of the collection's creator. */
    public EmailAddress creator = null;
    /** E-mail address of the collection's maintainer. */
    public EmailAddress maintainer = null;
    /** The collection configuration file. */
    public File in_file = null;
    /** A list of formating strings to use at build time. */
    public FormatManager formats = null;
    /** The manager in charge of displaying this manager and the controls for other managers. */
    public GUI gui = null;
    /** List of indexes to be built, and the default index. */
    public IndexManager indexes = null;
    /** Contains instructions dealing with the collection language. */
    public LanguageManager languages = null;
    /** A simple manager for the visual review of metadata sets. */
    public MetadataSetManager metadatasets = null;
    /** A list of plugins to use at build time. */
    public PlugInManager plugins = null;
    /** Contains: A list of subcollections, (defined on metadatadata), a list of which subcollection indexes to build and the default subcollection index. */
    public SubcollectionManager subcollections = null;
    /** A supercollection command allows a single search to be conducted across several collections. It is a very basic command and so avoids all the crazy model stuff that exists in most of the design managers. */
    public SuperCollectionManager superman; // Just cause I could ;p
    /** The text translation manager. */
    public TranslationManager transman;
    /** A list of whatever commands could not be parsed at all. */
    private ArrayList rest = null;
    /** A reference to the Gatherer. */
    private Gatherer gatherer = null;
    /** Constructor.
     * @param gatherer The <strong>Gatherer</strong> that created this class.
     * @see org.greenstone.gatherer.cdm.ClassifierManager
     * @see org.greenstone.gatherer.cdm.CollectionMetaManager
     * @see org.greenstone.gatherer.cdm.FormatManager
     * @see org.greenstone.gatherer.cdm.IndexManager
     * @see org.greenstone.gatherer.cdm.LanguageManager
     * @see org.greenstone.gatherer.cdm.MetadataSetManager
     * @see org.greenstone.gatherer.cdm.PlugInManager
     * @see org.greenstone.gatherer.cdm.SubcollectionManager
     */
    public CollectionDesignManager() {
	this.gatherer = Gatherer.self;
	this.classifiers = new ClassifierManager(gatherer, this);
	this.collectionmetadatum = new CollectionMetaManager(gatherer, this);
	this.formats = new FormatManager(gatherer, this);
	this.indexes = new IndexManager(gatherer, this);
	this.languages = new LanguageManager(gatherer, this);
	this.metadatasets = new MetadataSetManager(gatherer);
	this.plugins = new PlugInManager(gatherer, this);
	this.rest = new ArrayList();
	this.subcollections = new SubcollectionManager(gatherer, this);
	this.superman = new SuperCollectionManager();
	this.transman = new TranslationManager(this);
    }
    /** In order to prevent Components that have registered themselves wasting memory, this method invalidates each of the sub-managers controls, causing them to unregister listeners.
     * @see org.greenstone.gatherer.cdm.ClassifierManager
     * @see org.greenstone.gatherer.cdm.CollectionMetaManager
     * @see org.greenstone.gatherer.cdm.FormatManager
     * @see org.greenstone.gatherer.cdm.IndexManager
     * @see org.greenstone.gatherer.cdm.LanguageManager
     * @see org.greenstone.gatherer.cdm.MetadataSetManager
     * @see org.greenstone.gatherer.cdm.PlugInManager
     * @see org.greenstone.gatherer.cdm.SubcollectionManager
     */
    public void destroy() {
	// Remove visual the component from its parent.
	if(gui.getParent() != null) {
	    gui.getParent().remove(gui);
	}
	// Remove references from persistant listeners.
	if(classifiers != null && formats != null && gui != null && indexes != null && languages != null && metadatasets != null && plugins != null && subcollections != null) {
	    classifiers.invalidateControls();
	    formats.invalidateControls();
	    gui.invalidateControls();
	    indexes.invalidateControls();
	    languages.invalidateControls();
	    metadatasets.invalidateControls();
	    plugins.invalidateControls();
	    subcollections.invalidateControls();
	    superman.invalidateControls();
	}
	// Null globals.
	classifiers = null;
	collectionmetadatum = null;
	creator = null;
	maintainer = null;
	in_file = null;
	formats = null;
	gui = null;
	indexes = null;
	languages = null;
	metadatasets = null;
	plugins = null;
	subcollections = null;
	superman = null;
	rest = null;
	gatherer = null;
    }

    /** Display the GUI interface for the CollectionDesignManager in the centre of the indicated panel.
     * @param target The <strong>JPanel</strong> you wish to display the GUI on.
     * @see org.greenstone.gatherer.cdm.GUI
     */
    public void display(JPanel target) {
	this.gui = new GUI(gatherer, this);
	target.add(gui, BorderLayout.CENTER);
    }
    /** When the tab on the JTabbedPane that contains the GUI is selected, this method is called to ensure that the controls are all up to date, in terms of references to metadata etc.
     * @see org.greenstone.gatherer.cdm.GUI
     */
    public void gainFocus() {
	gui.updateUI();
    }
    /** Retrieve the current set of indexes as defined by the user configuration.
     * @return An <strong>ArrayList</strong> of indexes.
     * @see org.greenstone.gatherer.cdm.Index
     * @see org.greenstone.gatherer.cdm.IndexManager
     */
    public ArrayList getIndexes() {
	ArrayList result = new ArrayList();
	int size = indexes.size();
	for(int i = 0; i < size; i++) {
	    result.add(indexes.getIndex(i));
	}
	return result;
    }
    /** Retrieve the name of the collection configuration file which is being used as the source of the information in this object.
     * @return The files absolute path as a <strong>String</strong>.
     */
    public String getFilename() {
	return in_file.getAbsolutePath();
    }
    /** Used to parse the given file as if it was a collection configuration file, passing any parsed commands to the relevant submanager. Currently ignores any comments, and marks any other lines as being unrecognizable if they can't be parsed.
     * @param filename The name of the file you wish to attempt to pass as a <strong>String</strong>.
     * @see org.greenstone.gatherer.cdm.ClassifierManager
     * @see org.greenstone.gatherer.cdm.CollectionMetaManager
     * @see org.greenstone.gatherer.cdm.FormatManager
     * @see org.greenstone.gatherer.cdm.IndexManager
     * @see org.greenstone.gatherer.cdm.LanguageManager
     * @see org.greenstone.gatherer.cdm.MetadataSetManager
     * @see org.greenstone.gatherer.cdm.PlugInManager
     * @see org.greenstone.gatherer.cdm.SubcollectionManager
     * @see org.greenstone.gatherer.util.EmailAddress
     */
    private BufferedReader in;
    /** If one of the submanagers detects that it hasn't been provided enough arguments (such as format which always requires at least three) it can call this method to request another line of text from the collect.cfg. If the collect.cfg was not well formed this will cause some problems, but that can't be helped. 
     * @param prefix the String we have so far
     * @return a String with a further line form the collect.cfg appended
     */
    public String parseMore(String prefix) {
	///ystem.err.println("Command incomplete: " + prefix);
	try {
	    String suffix = in.readLine();
	    ///ystem.err.println("Adding: " + suffix);
	    if(suffix != null) {
		prefix = prefix + " " + suffix;
	    }
	    suffix = null;
	}
	catch(Exception error) {
	    error.printStackTrace();
	}
	return prefix;
    }

    public void parse(String filename) {
	try {
	    in_file = new File(filename);
	    FileReader in_reader = new FileReader(in_file);
	    in = new BufferedReader(in_reader);
	    String command = null;
	    while((command = in.readLine()) != null) {
		if(command.length() > 0) {
		    // Now we've finished parsing a command line, see what manager wants a piece of it.
		    // We have to test the end of command for the special character '\'. If found, remove it and append the next line, then repeat.
		    while(command.trim().endsWith("\\")) {
			String next_line = (in.readLine()).trim();
			if(next_line != null && next_line.length() > 0) {
			    command = command.substring(0, command.lastIndexOf("\\"));
			    command = command + next_line;
			}
		    }
		    // Now we've finished parsing a command line, see what manager wants a piece of it.
		    boolean found = false;
		    String command_lc = command.toLowerCase();
		    if(command_lc.startsWith("creator")) {
			creator = new EmailAddress(gatherer, command);
			found = true;
		    }
		    if(command_lc.startsWith("maintainer")) {
			maintainer = new EmailAddress(gatherer, command);
			found = true;
		    }
		    if(command_lc.startsWith("public")) {
			if(command_lc.endsWith("true")) {
			    public_col = true;
			}
			else {
			    public_col = false;
			}
			found = true;
		    }
		    if(command_lc.startsWith("beta")) {
			if(command_lc.endsWith("false")) {
			    beta = false;
			}
			else {
			    beta = true;
			}
			found = true;
		    }
		    if(!found) {
			found = indexes.parse(command);
		    }
		    if(!found) {
			found = subcollections.parse(command, false);
		    }
		    if(!found) {
			found = languages.parse(command);
		    }
		    if(!found) {
			found = plugins.parse(command);
		    }
		    if(!found) {
			found = classifiers.parse(command);
		    }
		    if(!found) {
			found = formats.parse(command, false);
		    }
		    if(!found) {
			found = collectionmetadatum.parse(command, false);
		    }
		    if(!found) {
			found = superman.parse(command);
		    }
		    // Metadataset commands
		    if(command_lc.startsWith("metadataset")) {
			// Nothing yet. Eventually used to import metadata.
			found = true;
		    }
		    // Comments we ignore.
		    if(command_lc.startsWith("# these instructions are not recognized by the gatherer.")) {
			// Ignore
			found = true;
		    }
		    // We have been unable to parse this command, add it to rest.
		    if(!found) {
			rest.add(command);
		    }
		}
	    }
	    in.close();
	    // Now attempt to finalize any commands that were not immediately parsed as they were waiting for unresolved references.
	    subcollections.reparseUnresolved();
	    classifiers.reparseUnresolved();
	    formats.reparseUnresolved();
	    collectionmetadatum.reparseUnresolved();
	}
	catch(Exception error) {
	    error.printStackTrace();
	}
    }
    /** Cause the current definitions within the Collection Design Manager to be written back out to whatever collect.cfg it is based upon, taking care to change reference pointers into something more sensible within the text file.
     * @see org.greenstone.gatherer.cdm.ClassifierManager
     * @see org.greenstone.gatherer.cdm.CollectionMetaManager
     * @see org.greenstone.gatherer.cdm.FormatManager
     * @see org.greenstone.gatherer.cdm.IndexManager
     * @see org.greenstone.gatherer.cdm.LanguageManager
     * @see org.greenstone.gatherer.cdm.MetadataSetManager
     * @see org.greenstone.gatherer.cdm.PlugInManager
     * @see org.greenstone.gatherer.cdm.SubcollectionManager
     * @see org.greenstone.gatherer.util.EmailAddress
     */
    public void save() {
	try {
	    // If the file already exists (it should) rename it.
	    if(in_file.exists()) {
		String filename = in_file.getAbsolutePath();
		File backup = new File(filename + "~");
		backup.deleteOnExit();
		if(!in_file.renameTo(backup)) {
		    Gatherer.println("Error in CollectionDesignManager.parse(): FileRenamedException");
		}
		in_file = new File(filename); // Just in case we moved it.
	    }
	    FileOutputStream out = new FileOutputStream(in_file);

	    StringBuffer text = new StringBuffer("");
	    if(creator != null) {
		text.append(creator.toString());
		text.append("\n");
	    }
	    if(maintainer != null) {
		text.append(maintainer.toString());
		text.append("\n");
	    }
	    if(public_col) {
		text.append("public true\n");
	    }
	    else {
		text.append("public false\n");
	    }
	    if(beta) {
		text.append("beta true\n");
	    }
	    else {
		text.append("beta false\n");
	    }
	    if(text.length() > 0) {
		text.append("\n");
	    }
	    if(plugins.size() > 0) {
		text.append(plugins.toString());
	    }
	    if(indexes.size() > 0) {
		text.append(indexes.toString());
	    }
	    if(languages.size() > 0) {
		text.append(languages.toString());
	    }
	    if(subcollections.size() > 0) {
		text.append(subcollections.toString());
	    }
	    String super_collections_command = superman.toString();
	    if(super_collections_command != null) {
		text.append(super_collections_command);
	    }
	    if(classifiers.size() > 0) {
		text.append(classifiers.toString());
	    }
	    if(formats.size() > 0) {
		text.append(formats.toString());
	    }
	    if(collectionmetadatum.size() > 0) {
		text.append(collectionmetadatum.toString());
	    }
	    text.append(metadatasets.toString());
	    if(rest.size() > 0) {
		// Write out rest at the bottom.
		text.append("# These instructions are not recognized by the Gatherer.\n");
		for(int i = 0; i < rest.size(); i++) {
		    text.append(rest.get(i));
		    text.append("\n");
		}
	    }
	    out.write(text.toString().getBytes());
	    out.close();
	    out = null;
	}
	catch(Exception error) {
	    error.printStackTrace();
	}
    }
    /** Method used during a global search and replace to highlight the appropriate record within the Collection Design Managers version of the Metadata Set Manager (view only).
     * @param element The name of the desired element as a <strong>String</strong>.
     * @see org.greenstone.gatherer.cdm.GUI
     * @see org.greenstone.gatherer.cdm.MetadataSetManager
     */
    public Rectangle setSelectedElement(String element) {
	// First ensure that the metadata set controls are visible.
	gui.setSelectedView("CDM.GUI.MetadataSets");
	// Then tell them to select the given element.
	return metadatasets.setSelectedElement(element);
    }
}
